"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var core_1 = require('angular2/core');
var dropdown_service_1 = require('./dropdown.service');
var Dropdown = (function () {
    function Dropdown(el) {
        this.el = el;
        this.onToggle = new core_1.EventEmitter(false);
        this.isOpenChange = new core_1.EventEmitter(false);
        this.isDisabled = false;
        this._dropdownService = new dropdown_service_1.DropdownService();
        this._selectedItem = null;
    }
    Object.defineProperty(Dropdown.prototype, "isOpen", {
        get: function () {
            return this._isOpen;
        },
        set: function (value) {
            var _this = this;
            if (this.isDisabled) {
                value = false;
            }
            this._isOpen = !!value;
            if (this.isOpen) {
                this._dropdownService.open(this);
                this.selectedItem = null;
            }
            else {
                this._dropdownService.close();
            }
            setTimeout(function () {
                _this.onToggle.emit(_this._isOpen);
                _this.isOpenChange.emit(_this._isOpen);
            });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Dropdown.prototype, "dropDownMenu", {
        set: function (dropdownMenu) {
            // init drop down menu
            this.menuEl = dropdownMenu.el;
        },
        enumerable: true,
        configurable: true
    });
    Dropdown.prototype.toggle = function () {
        this.isOpen = !this.isOpen;
    };
    Dropdown.prototype.click = function (event) {
        event.stopPropagation();
        if (!this.menuEl.nativeElement.contains(event.target)) {
            this.toggle();
        }
        return false;
    };
    Dropdown.prototype.keyPress = function (keyCode) {
        //noinspection FallThroughInSwitchStatementJS
        switch (keyCode) {
            case dropdown_service_1.KEYCODE.DOWN:
                this.selectNextItem();
                break;
            case dropdown_service_1.KEYCODE.UP:
                this.selectPreviousItem();
                break;
            case dropdown_service_1.KEYCODE.ENTER:
                if (!this.selectedItem.hasAttribute("suiDropdown")) {
                    this.selectedItem.click();
                    this.isOpen = false;
                    break;
                }
            //Fall through on purpose! (So enter on a nested dropdown acts as right arrow)
            case dropdown_service_1.KEYCODE.RIGHT:
                if (this.selectedItem.hasAttribute("suiDropdown")) {
                    this.selectedItem.click();
                    this.selectedItem = this.selectedItem.querySelector(".item:not(.disabled)");
                }
                break;
            case dropdown_service_1.KEYCODE.LEFT:
                if (this.selectedItem.parentElement != this.menuEl.nativeElement) {
                    console.log(this.selectedItem.parentElement.parentElement);
                    this.selectedItem.parentElement.parentElement.click();
                    this.selectedItem = this.selectedItem.parentElement.parentElement;
                }
                break;
        }
    };
    Object.defineProperty(Dropdown.prototype, "selectedItem", {
        get: function () {
            return this._selectedItem;
        },
        set: function (item) {
            if (this._selectedItem) {
                this._selectedItem.classList.remove("selected");
            }
            this._selectedItem = item;
            if (item) {
                item.classList.add("selected");
            }
        },
        enumerable: true,
        configurable: true
    });
    Dropdown.prototype.selectNextItem = function () {
        if (!this.selectedItem) {
            this.selectedItem = this.menuEl.nativeElement.querySelector(".item:not(.disabled)");
            return;
        }
        var nextItem = this.selectedItem.nextElementSibling;
        if (nextItem) {
            this.selectedItem = nextItem;
            if (this.selectedItem.classList.contains("disabled")) {
                this.selectNextItem();
            }
        }
    };
    Dropdown.prototype.selectPreviousItem = function () {
        var previousItem = this.selectedItem.previousElementSibling;
        if (previousItem) {
            this.selectedItem = previousItem;
            if (this.selectedItem.classList.contains("disabled")) {
                this.selectPreviousItem();
            }
        }
    };
    __decorate([
        core_1.HostBinding('class.visible'),
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], Dropdown.prototype, "isOpen", null);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', String)
    ], Dropdown.prototype, "autoClose", void 0);
    __decorate([
        core_1.Output(), 
        __metadata('design:type', core_1.EventEmitter)
    ], Dropdown.prototype, "onToggle", void 0);
    __decorate([
        core_1.Output(), 
        __metadata('design:type', core_1.EventEmitter)
    ], Dropdown.prototype, "isOpenChange", void 0);
    __decorate([
        core_1.HostBinding('class.disabled'),
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], Dropdown.prototype, "isDisabled", void 0);
    __decorate([
        core_1.HostListener('click', ['$event']), 
        __metadata('design:type', Function), 
        __metadata('design:paramtypes', [MouseEvent]), 
        __metadata('design:returntype', Boolean)
    ], Dropdown.prototype, "click", null);
    Dropdown = __decorate([
        core_1.Directive({
            selector: '[suiDropdown]'
        }), 
        __metadata('design:paramtypes', [core_1.ElementRef])
    ], Dropdown);
    return Dropdown;
}());
exports.Dropdown = Dropdown;
//# sourceMappingURL=dropdown.directive.js.map