"use strict";
exports.DISABLED = 'disabled';
exports.OUTSIDECLICK = 'outsideClick';
exports.KEYCODE = {
    LEFT: 37,
    UP: 38,
    RIGHT: 39,
    DOWN: 40,
    ESCAPE: 27,
    ENTER: 13
};
var DropdownService = (function () {
    function DropdownService() {
        this.closeDropdownBind = this.closeDropdown.bind(this);
        this.keybindFilterBind = this.keybindFilter.bind(this);
    }
    DropdownService.prototype.open = function (dropdown) {
        if (!this.dropdown) {
            window.document.addEventListener('click', this.closeDropdownBind, true);
            if (!dropdown.el.nativeElement.parentElement.hasAttribute("suiDropdownMenu")) {
                window.document.addEventListener('keydown', this.keybindFilterBind);
            }
        }
        this.dropdown = dropdown;
    };
    DropdownService.prototype.close = function () {
        this.dropdown = null;
        window.document.removeEventListener('click', this.closeDropdownBind, true);
        window.document.removeEventListener('keydown', this.keybindFilterBind);
    };
    DropdownService.prototype.closeDropdown = function (event) {
        console.log("Hi");
        //Never close the dropdown if autoClose is disabled
        if (event && this.dropdown.autoClose === exports.DISABLED) {
            return;
        }
        //Don't close the dropdown when clicking the toggle
        if (event && this.dropdown.el.nativeElement.contains(event.target) &&
            !this.dropdown.menuEl.nativeElement.contains(event.target)) {
            return;
        }
        //Don't close the dropdown if expanding a nested dropdown
        if (event && this.dropdown.menuEl.nativeElement.contains(event.target) &&
            event.target.hasAttribute("suiDropdown")) {
            return;
        }
        //Don't close the dropdown if clicking on any input element
        if (event && this.dropdown.menuEl &&
            /input|textarea/i.test(event.target.tagName) &&
            this.dropdown.menuEl.nativeElement.contains(event.target)) {
            return;
        }
        //Don't close the dropdown when clicking inside if autoClose is outsideClick
        if (event && this.dropdown.autoClose === exports.OUTSIDECLICK &&
            this.dropdown.menuEl &&
            this.dropdown.menuEl.nativeElement.contains(event.target)) {
            return;
        }
        //Close the dropdown
        this.dropdown.isOpen = false;
    };
    DropdownService.prototype.keybindFilter = function (event) {
        if (event.which === exports.KEYCODE.ESCAPE) {
            this.dropdown.isOpen = false;
            return;
        }
        if (this.dropdown.isOpen &&
            ([exports.KEYCODE.ENTER, exports.KEYCODE.UP, exports.KEYCODE.RIGHT, exports.KEYCODE.DOWN, exports.KEYCODE.LEFT]
                .find(function (keyCode) { return event.which == keyCode; }))) {
            event.preventDefault();
            event.stopPropagation();
            this.dropdown.keyPress(event.which);
        }
    };
    return DropdownService;
}());
exports.DropdownService = DropdownService;
//# sourceMappingURL=dropdown.service.js.map